#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="ai_name_api.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

from __future__ import absolute_import

from AsposeEmailCloudSdk.api.api_base import ApiBase
from AsposeEmailCloudSdk.models import *


class AiNameApi(ApiBase):
    """
    Aspose.Email Cloud API. AiNameApi operations.

    """

    def __init__(self, api_client):
        super(AiNameApi, self).__init__(api_client)

    def complete(self, request: AiNameCompleteRequest) -> AiNameWeightedVariants:
        """The call proposes k most probable names for given starting characters.             


        :param request: AiNameCompleteRequest object with parameters
        :type request: AiNameCompleteRequest
        :return: AiNameWeightedVariants
        """
        # verify the required parameter 'name' is set
        if request.name is None:
            raise ValueError("Missing the required parameter `name` when calling `complete`")

        collection_formats = {}
        path = '/email/AiName/complete'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('name') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.name if request.name is not None else '')
        else:
            if request.name is not None:
                query_params.append((self._lowercase_first_letter('name'), request.name))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameWeightedVariants')

    def expand(self, request: AiNameExpandRequest) -> AiNameWeightedVariants:
        """Expands a person&#39;s name into a list of possible alternatives using options for expanding instructions.             


        :param request: AiNameExpandRequest object with parameters
        :type request: AiNameExpandRequest
        :return: AiNameWeightedVariants
        """
        # verify the required parameter 'name' is set
        if request.name is None:
            raise ValueError("Missing the required parameter `name` when calling `expand`")

        collection_formats = {}
        path = '/email/AiName/expand'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('name') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.name if request.name is not None else '')
        else:
            if request.name is not None:
                query_params.append((self._lowercase_first_letter('name'), request.name))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameWeightedVariants')

    def expand_parsed(self, request: AiNameParsedRequest) -> AiNameWeightedVariants:
        """Expands a person&#39;s parsed name into a list of possible alternatives using options for expanding instructions.             

        :param request: Parsed name with options.
        :type request: AiNameParsedRequest
        :return: AiNameWeightedVariants
        """
        # verify the required parameter 'request' is set
        if request is None:
            raise ValueError("Missing the required parameter `request` when calling `expand_parsed`")

        collection_formats = {}
        path = '/email/AiName/expand-parsed'

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])
        body_params = request
        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, None, None, header_params, None, body_params, None, None, auth_settings)

        return self._make_request(http_request_object, 'PUT', 'AiNameWeightedVariants')

    def format(self, request: AiNameFormatRequest) -> AiNameFormatted:
        """Formats a person&#39;s name in correct case and name order using options for formatting instructions.             


        :param request: AiNameFormatRequest object with parameters
        :type request: AiNameFormatRequest
        :return: AiNameFormatted
        """
        # verify the required parameter 'name' is set
        if request.name is None:
            raise ValueError("Missing the required parameter `name` when calling `format`")

        collection_formats = {}
        path = '/email/AiName/format'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('name') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.name if request.name is not None else '')
        else:
            if request.name is not None:
                query_params.append((self._lowercase_first_letter('name'), request.name))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('format') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.format if request.format is not None else '')
        else:
            if request.format is not None:
                query_params.append((self._lowercase_first_letter('format'), request.format))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameFormatted')

    def format_parsed(self, request: AiNameParsedRequest) -> AiNameFormatted:
        """Formats a person&#39;s parsed name in correct case and name order using options for formatting instructions.             

        :param request: Parsed name with options.
        :type request: AiNameParsedRequest
        :return: AiNameFormatted
        """
        # verify the required parameter 'request' is set
        if request is None:
            raise ValueError("Missing the required parameter `request` when calling `format_parsed`")

        collection_formats = {}
        path = '/email/AiName/format-parsed'

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])
        body_params = request
        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, None, None, header_params, None, body_params, None, None, auth_settings)

        return self._make_request(http_request_object, 'PUT', 'AiNameFormatted')

    def genderize(self, request: AiNameGenderizeRequest) -> AiNameGenderHypothesisList:
        """Detect person&#39;s gender from name string.             


        :param request: AiNameGenderizeRequest object with parameters
        :type request: AiNameGenderizeRequest
        :return: AiNameGenderHypothesisList
        """
        # verify the required parameter 'name' is set
        if request.name is None:
            raise ValueError("Missing the required parameter `name` when calling `genderize`")

        collection_formats = {}
        path = '/email/AiName/genderize'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('name') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.name if request.name is not None else '')
        else:
            if request.name is not None:
                query_params.append((self._lowercase_first_letter('name'), request.name))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameGenderHypothesisList')

    def genderize_parsed(self, request: AiNameParsedRequest) -> AiNameGenderHypothesisList:
        """Detect person&#39;s gender from parsed name.             

        :param request: Gender detection request data.
        :type request: AiNameParsedRequest
        :return: AiNameGenderHypothesisList
        """
        # verify the required parameter 'request' is set
        if request is None:
            raise ValueError("Missing the required parameter `request` when calling `genderize_parsed`")

        collection_formats = {}
        path = '/email/AiName/genderize-parsed'

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])
        body_params = request
        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, None, None, header_params, None, body_params, None, None, auth_settings)

        return self._make_request(http_request_object, 'PUT', 'AiNameGenderHypothesisList')

    def match(self, request: AiNameMatchRequest) -> AiNameMatchResult:
        """Compare people&#39;s names. Uses options for comparing instructions.             


        :param request: AiNameMatchRequest object with parameters
        :type request: AiNameMatchRequest
        :return: AiNameMatchResult
        """
        # verify the required parameter 'name' is set
        if request.name is None:
            raise ValueError("Missing the required parameter `name` when calling `match`")
        # verify the required parameter 'other_name' is set
        if request.other_name is None:
            raise ValueError("Missing the required parameter `other_name` when calling `match`")

        collection_formats = {}
        path = '/email/AiName/match'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('name') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.name if request.name is not None else '')
        else:
            if request.name is not None:
                query_params.append((self._lowercase_first_letter('name'), request.name))
        path_parameter = '{' + self._lowercase_first_letter('otherName') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.other_name if request.other_name is not None else '')
        else:
            if request.other_name is not None:
                query_params.append((self._lowercase_first_letter('otherName'), request.other_name))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameMatchResult')

    def match_parsed(self, request: AiNameMatchParsedRequest) -> AiNameMatchResult:
        """Compare people&#39;s parsed names and attributes. Uses options for comparing instructions.             

        :param request: Parsed names to match.
        :type request: AiNameMatchParsedRequest
        :return: AiNameMatchResult
        """
        # verify the required parameter 'request' is set
        if request is None:
            raise ValueError("Missing the required parameter `request` when calling `match_parsed`")

        collection_formats = {}
        path = '/email/AiName/match-parsed'

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])
        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])
        body_params = request
        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, None, None, header_params, None, body_params, None, None, auth_settings)

        return self._make_request(http_request_object, 'PUT', 'AiNameMatchResult')

    def parse(self, request: AiNameParseRequest) -> AiNameComponentList:
        """Parse name to components.             


        :param request: AiNameParseRequest object with parameters
        :type request: AiNameParseRequest
        :return: AiNameComponentList
        """
        # verify the required parameter 'name' is set
        if request.name is None:
            raise ValueError("Missing the required parameter `name` when calling `parse`")

        collection_formats = {}
        path = '/email/AiName/parse'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('name') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.name if request.name is not None else '')
        else:
            if request.name is not None:
                query_params.append((self._lowercase_first_letter('name'), request.name))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameComponentList')

    def parse_email_address(self, request: AiNameParseEmailAddressRequest) -> AiNameExtractedList:
        """Parse person&#39;s name out of an email address.             


        :param request: AiNameParseEmailAddressRequest object with parameters
        :type request: AiNameParseEmailAddressRequest
        :return: AiNameExtractedList
        """
        # verify the required parameter 'email_address' is set
        if request.email_address is None:
            raise ValueError("Missing the required parameter `email_address` when calling `parse_email_address`")

        collection_formats = {}
        path = '/email/AiName/parse-email-address'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('emailAddress') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.email_address if request.email_address is not None else '')
        else:
            if request.email_address is not None:
                query_params.append((self._lowercase_first_letter('emailAddress'), request.email_address))
        path_parameter = '{' + self._lowercase_first_letter('language') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.language if request.language is not None else '')
        else:
            if request.language is not None:
                query_params.append((self._lowercase_first_letter('language'), request.language))
        path_parameter = '{' + self._lowercase_first_letter('location') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.location if request.location is not None else '')
        else:
            if request.location is not None:
                query_params.append((self._lowercase_first_letter('location'), request.location))
        path_parameter = '{' + self._lowercase_first_letter('encoding') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.encoding if request.encoding is not None else '')
        else:
            if request.encoding is not None:
                query_params.append((self._lowercase_first_letter('encoding'), request.encoding))
        path_parameter = '{' + self._lowercase_first_letter('script') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.script if request.script is not None else '')
        else:
            if request.script is not None:
                query_params.append((self._lowercase_first_letter('script'), request.script))
        path_parameter = '{' + self._lowercase_first_letter('style') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.style if request.style is not None else '')
        else:
            if request.style is not None:
                query_params.append((self._lowercase_first_letter('style'), request.style))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'AiNameExtractedList')




#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="MapiCalendarExceptionInfoDto.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

import pprint
import re
import six
from typing import List, Set, Dict, Tuple, Optional
from datetime import datetime

from AsposeEmailCloudSdk.models.mapi_attachment_dto import MapiAttachmentDto


class MapiCalendarExceptionInfoDto(object):
    """An exception specifies changes to an instance of a recurring series.             
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'attachments': 'list[MapiAttachmentDto]',
        'body': 'str',
        'busy_status': 'str',
        'end_date_time': 'datetime',
        'has_attachment': 'bool',
        'location': 'str',
        'meeting_type': 'str',
        'original_start_date': 'datetime',
        'override_flags': 'list[str]',
        'reminder_delta': 'int',
        'reminder_set': 'bool',
        'start_date_time': 'datetime',
        'subject': 'str',
        'sub_type': 'int'
    }

    attribute_map = {
        'attachments': 'attachments',
        'body': 'body',
        'busy_status': 'busyStatus',
        'end_date_time': 'endDateTime',
        'has_attachment': 'hasAttachment',
        'location': 'location',
        'meeting_type': 'meetingType',
        'original_start_date': 'originalStartDate',
        'override_flags': 'overrideFlags',
        'reminder_delta': 'reminderDelta',
        'reminder_set': 'reminderSet',
        'start_date_time': 'startDateTime',
        'subject': 'subject',
        'sub_type': 'subType'
    }

    def __init__(self, attachments: List[MapiAttachmentDto] = None, body: str = None, busy_status: str = None, end_date_time: datetime = None, has_attachment: bool = None, location: str = None, meeting_type: str = None, original_start_date: datetime = None, override_flags: List[str] = None, reminder_delta: int = None, reminder_set: bool = None, start_date_time: datetime = None, subject: str = None, sub_type: int = None):
        """
        An exception specifies changes to an instance of a recurring series.             
        :param attachments: Attachments in the recurrence exception.             
        :type attachments: List[MapiAttachmentDto]
        :param body: Body.             
        :type body: str
        :param busy_status: Enumerates the mapi calendar possible busy status Enum, available values: Free, Tentative, Busy, OutOfOffice
        :type busy_status: str
        :param end_date_time: End date.             
        :type end_date_time: datetime
        :param has_attachment: Value of this field specifies whether the Exception Embedded Message object contains attachments.             
        :type has_attachment: bool
        :param location: Location.             
        :type location: str
        :param meeting_type: Enumerates the appointment state Enum, available values: Meeting, Received, Canceled
        :type meeting_type: str
        :param original_start_date: Original start date.             
        :type original_start_date: datetime
        :param override_flags: Override flags.             
        :type override_flags: List[str]
        :param reminder_delta: Reminder delta.             
        :type reminder_delta: int
        :param reminder_set: Value for the PidLidReminderSet property.             
        :type reminder_set: bool
        :param start_date_time: Start date.             
        :type start_date_time: datetime
        :param subject: Subject.             
        :type subject: str
        :param sub_type: SubType.             
        :type sub_type: int
        """

        self._attachments = None
        self._body = None
        self._busy_status = None
        self._end_date_time = None
        self._has_attachment = None
        self._location = None
        self._meeting_type = None
        self._original_start_date = None
        self._override_flags = None
        self._reminder_delta = None
        self._reminder_set = None
        self._start_date_time = None
        self._subject = None
        self._sub_type = None

        if attachments is not None:
            self.attachments = attachments
        if body is not None:
            self.body = body
        if busy_status is not None:
            self.busy_status = busy_status
        if end_date_time is not None:
            self.end_date_time = end_date_time
        if has_attachment is not None:
            self.has_attachment = has_attachment
        if location is not None:
            self.location = location
        if meeting_type is not None:
            self.meeting_type = meeting_type
        if original_start_date is not None:
            self.original_start_date = original_start_date
        if override_flags is not None:
            self.override_flags = override_flags
        if reminder_delta is not None:
            self.reminder_delta = reminder_delta
        if reminder_set is not None:
            self.reminder_set = reminder_set
        if start_date_time is not None:
            self.start_date_time = start_date_time
        if subject is not None:
            self.subject = subject
        if sub_type is not None:
            self.sub_type = sub_type


    @property
    def attachments(self) -> List[MapiAttachmentDto]:
        """
        Attachments in the recurrence exception.             

        :return: The attachments of this MapiCalendarExceptionInfoDto.
        :rtype: list[MapiAttachmentDto]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments: List[MapiAttachmentDto]):
        """
        Attachments in the recurrence exception.             

        :param attachments: The attachments of this MapiCalendarExceptionInfoDto.
        :type: list[MapiAttachmentDto]
        """
        self._attachments = attachments

    @property
    def body(self) -> str:
        """
        Body.             

        :return: The body of this MapiCalendarExceptionInfoDto.
        :rtype: str
        """
        return self._body

    @body.setter
    def body(self, body: str):
        """
        Body.             

        :param body: The body of this MapiCalendarExceptionInfoDto.
        :type: str
        """
        self._body = body

    @property
    def busy_status(self) -> str:
        """
        Enumerates the mapi calendar possible busy status Enum, available values: Free, Tentative, Busy, OutOfOffice

        :return: The busy_status of this MapiCalendarExceptionInfoDto.
        :rtype: str
        """
        return self._busy_status

    @busy_status.setter
    def busy_status(self, busy_status: str):
        """
        Enumerates the mapi calendar possible busy status Enum, available values: Free, Tentative, Busy, OutOfOffice

        :param busy_status: The busy_status of this MapiCalendarExceptionInfoDto.
        :type: str
        """
        if busy_status is None:
            raise ValueError("Invalid value for `busy_status`, must not be `None`")
        self._busy_status = busy_status

    @property
    def end_date_time(self) -> datetime:
        """
        End date.             

        :return: The end_date_time of this MapiCalendarExceptionInfoDto.
        :rtype: datetime
        """
        return self._end_date_time

    @end_date_time.setter
    def end_date_time(self, end_date_time: datetime):
        """
        End date.             

        :param end_date_time: The end_date_time of this MapiCalendarExceptionInfoDto.
        :type: datetime
        """
        if end_date_time is None:
            raise ValueError("Invalid value for `end_date_time`, must not be `None`")
        self._end_date_time = end_date_time

    @property
    def has_attachment(self) -> bool:
        """
        Value of this field specifies whether the Exception Embedded Message object contains attachments.             

        :return: The has_attachment of this MapiCalendarExceptionInfoDto.
        :rtype: bool
        """
        return self._has_attachment

    @has_attachment.setter
    def has_attachment(self, has_attachment: bool):
        """
        Value of this field specifies whether the Exception Embedded Message object contains attachments.             

        :param has_attachment: The has_attachment of this MapiCalendarExceptionInfoDto.
        :type: bool
        """
        if has_attachment is None:
            raise ValueError("Invalid value for `has_attachment`, must not be `None`")
        self._has_attachment = has_attachment

    @property
    def location(self) -> str:
        """
        Location.             

        :return: The location of this MapiCalendarExceptionInfoDto.
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location: str):
        """
        Location.             

        :param location: The location of this MapiCalendarExceptionInfoDto.
        :type: str
        """
        self._location = location

    @property
    def meeting_type(self) -> str:
        """
        Enumerates the appointment state Enum, available values: Meeting, Received, Canceled

        :return: The meeting_type of this MapiCalendarExceptionInfoDto.
        :rtype: str
        """
        return self._meeting_type

    @meeting_type.setter
    def meeting_type(self, meeting_type: str):
        """
        Enumerates the appointment state Enum, available values: Meeting, Received, Canceled

        :param meeting_type: The meeting_type of this MapiCalendarExceptionInfoDto.
        :type: str
        """
        if meeting_type is None:
            raise ValueError("Invalid value for `meeting_type`, must not be `None`")
        self._meeting_type = meeting_type

    @property
    def original_start_date(self) -> datetime:
        """
        Original start date.             

        :return: The original_start_date of this MapiCalendarExceptionInfoDto.
        :rtype: datetime
        """
        return self._original_start_date

    @original_start_date.setter
    def original_start_date(self, original_start_date: datetime):
        """
        Original start date.             

        :param original_start_date: The original_start_date of this MapiCalendarExceptionInfoDto.
        :type: datetime
        """
        if original_start_date is None:
            raise ValueError("Invalid value for `original_start_date`, must not be `None`")
        self._original_start_date = original_start_date

    @property
    def override_flags(self) -> List[str]:
        """
        Override flags.              Items: Specifies what data in the MapiCalendarOverride structure has a value different from the recurring series. Enum, available values: Subject, MeetingType, ReminderDelta, Reminder, Location, BusyStatus, Attachment, Subtype, AppointmentColor, ExceptionalBody

        :return: The override_flags of this MapiCalendarExceptionInfoDto.
        :rtype: list[str]
        """
        return self._override_flags

    @override_flags.setter
    def override_flags(self, override_flags: List[str]):
        """
        Override flags.              Items: Specifies what data in the MapiCalendarOverride structure has a value different from the recurring series. Enum, available values: Subject, MeetingType, ReminderDelta, Reminder, Location, BusyStatus, Attachment, Subtype, AppointmentColor, ExceptionalBody

        :param override_flags: The override_flags of this MapiCalendarExceptionInfoDto.
        :type: list[str]
        """
        self._override_flags = override_flags

    @property
    def reminder_delta(self) -> int:
        """
        Reminder delta.             

        :return: The reminder_delta of this MapiCalendarExceptionInfoDto.
        :rtype: int
        """
        return self._reminder_delta

    @reminder_delta.setter
    def reminder_delta(self, reminder_delta: int):
        """
        Reminder delta.             

        :param reminder_delta: The reminder_delta of this MapiCalendarExceptionInfoDto.
        :type: int
        """
        if reminder_delta is None:
            raise ValueError("Invalid value for `reminder_delta`, must not be `None`")
        self._reminder_delta = reminder_delta

    @property
    def reminder_set(self) -> bool:
        """
        Value for the PidLidReminderSet property.             

        :return: The reminder_set of this MapiCalendarExceptionInfoDto.
        :rtype: bool
        """
        return self._reminder_set

    @reminder_set.setter
    def reminder_set(self, reminder_set: bool):
        """
        Value for the PidLidReminderSet property.             

        :param reminder_set: The reminder_set of this MapiCalendarExceptionInfoDto.
        :type: bool
        """
        if reminder_set is None:
            raise ValueError("Invalid value for `reminder_set`, must not be `None`")
        self._reminder_set = reminder_set

    @property
    def start_date_time(self) -> datetime:
        """
        Start date.             

        :return: The start_date_time of this MapiCalendarExceptionInfoDto.
        :rtype: datetime
        """
        return self._start_date_time

    @start_date_time.setter
    def start_date_time(self, start_date_time: datetime):
        """
        Start date.             

        :param start_date_time: The start_date_time of this MapiCalendarExceptionInfoDto.
        :type: datetime
        """
        if start_date_time is None:
            raise ValueError("Invalid value for `start_date_time`, must not be `None`")
        self._start_date_time = start_date_time

    @property
    def subject(self) -> str:
        """
        Subject.             

        :return: The subject of this MapiCalendarExceptionInfoDto.
        :rtype: str
        """
        return self._subject

    @subject.setter
    def subject(self, subject: str):
        """
        Subject.             

        :param subject: The subject of this MapiCalendarExceptionInfoDto.
        :type: str
        """
        self._subject = subject

    @property
    def sub_type(self) -> int:
        """
        SubType.             

        :return: The sub_type of this MapiCalendarExceptionInfoDto.
        :rtype: int
        """
        return self._sub_type

    @sub_type.setter
    def sub_type(self, sub_type: int):
        """
        SubType.             

        :param sub_type: The sub_type of this MapiCalendarExceptionInfoDto.
        :type: int
        """
        if sub_type is None:
            raise ValueError("Invalid value for `sub_type`, must not be `None`")
        self._sub_type = sub_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MapiCalendarExceptionInfoDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="ContactDto.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

import pprint
import re
import six
from typing import List, Set, Dict, Tuple, Optional
from datetime import datetime

from AsposeEmailCloudSdk.models.associated_person import AssociatedPerson
from AsposeEmailCloudSdk.models.attachment import Attachment
from AsposeEmailCloudSdk.models.contact_photo import ContactPhoto
from AsposeEmailCloudSdk.models.customer_event import CustomerEvent
from AsposeEmailCloudSdk.models.email_address import EmailAddress
from AsposeEmailCloudSdk.models.instant_messenger_address import InstantMessengerAddress
from AsposeEmailCloudSdk.models.phone_number import PhoneNumber
from AsposeEmailCloudSdk.models.postal_address import PostalAddress
from AsposeEmailCloudSdk.models.url import Url


class ContactDto(object):
    """VCard document representation.             
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'associated_persons': 'list[AssociatedPerson]',
        'attachments': 'list[Attachment]',
        'company_name': 'str',
        'computer_network_name': 'str',
        'customer_id': 'str',
        'department_name': 'str',
        'display_name': 'str',
        'email_addresses': 'list[EmailAddress]',
        'events': 'list[CustomerEvent]',
        'file_as': 'str',
        'file_as_mapping': 'str',
        'free_busy_location': 'str',
        'gender': 'str',
        'given_name': 'str',
        'government_id_number': 'str',
        'hobbies': 'str',
        'initials': 'str',
        'instant_messengers': 'list[InstantMessengerAddress]',
        'job_title': 'str',
        'language': 'str',
        'location': 'str',
        'middle_name': 'str',
        'nickname': 'str',
        'notes': 'str',
        'notes_format': 'str',
        'office_location': 'str',
        'organizational_id_number': 'str',
        'phone_numbers': 'list[PhoneNumber]',
        'photo': 'ContactPhoto',
        'physical_addresses': 'list[PostalAddress]',
        'preferred_text_encoding': 'str',
        'prefix': 'str',
        'profession': 'str',
        'suffix': 'str',
        'surname': 'str',
        'urls': 'list[Url]'
    }

    attribute_map = {
        'associated_persons': 'associatedPersons',
        'attachments': 'attachments',
        'company_name': 'companyName',
        'computer_network_name': 'computerNetworkName',
        'customer_id': 'customerId',
        'department_name': 'departmentName',
        'display_name': 'displayName',
        'email_addresses': 'emailAddresses',
        'events': 'events',
        'file_as': 'fileAs',
        'file_as_mapping': 'fileAsMapping',
        'free_busy_location': 'freeBusyLocation',
        'gender': 'gender',
        'given_name': 'givenName',
        'government_id_number': 'governmentIdNumber',
        'hobbies': 'hobbies',
        'initials': 'initials',
        'instant_messengers': 'instantMessengers',
        'job_title': 'jobTitle',
        'language': 'language',
        'location': 'location',
        'middle_name': 'middleName',
        'nickname': 'nickname',
        'notes': 'notes',
        'notes_format': 'notesFormat',
        'office_location': 'officeLocation',
        'organizational_id_number': 'organizationalIdNumber',
        'phone_numbers': 'phoneNumbers',
        'photo': 'photo',
        'physical_addresses': 'physicalAddresses',
        'preferred_text_encoding': 'preferredTextEncoding',
        'prefix': 'prefix',
        'profession': 'profession',
        'suffix': 'suffix',
        'surname': 'surname',
        'urls': 'urls'
    }

    def __init__(self, associated_persons: List[AssociatedPerson] = None, attachments: List[Attachment] = None, company_name: str = None, computer_network_name: str = None, customer_id: str = None, department_name: str = None, display_name: str = None, email_addresses: List[EmailAddress] = None, events: List[CustomerEvent] = None, file_as: str = None, file_as_mapping: str = None, free_busy_location: str = None, gender: str = None, given_name: str = None, government_id_number: str = None, hobbies: str = None, initials: str = None, instant_messengers: List[InstantMessengerAddress] = None, job_title: str = None, language: str = None, location: str = None, middle_name: str = None, nickname: str = None, notes: str = None, notes_format: str = None, office_location: str = None, organizational_id_number: str = None, phone_numbers: List[PhoneNumber] = None, photo: ContactPhoto = None, physical_addresses: List[PostalAddress] = None, preferred_text_encoding: str = None, prefix: str = None, profession: str = None, suffix: str = None, surname: str = None, urls: List[Url] = None):
        """
        VCard document representation.             
        :param associated_persons: Associated persons.             
        :type associated_persons: List[AssociatedPerson]
        :param attachments: Document attachments.             
        :type attachments: List[Attachment]
        :param company_name: Company name.             
        :type company_name: str
        :param computer_network_name: Computer network.             
        :type computer_network_name: str
        :param customer_id: Customer id.             
        :type customer_id: str
        :param department_name: Department name.             
        :type department_name: str
        :param display_name: Display name.             
        :type display_name: str
        :param email_addresses: Person's email addresses.             
        :type email_addresses: List[EmailAddress]
        :param events: Person's events.             
        :type events: List[CustomerEvent]
        :param file_as: A name used for sorting.             
        :type file_as: str
        :param file_as_mapping: Specifies how to generate and recompute the value of the dispidFileAs property when other contact name properties change. Coincides MS-OXPROPS revision 16.2 from 7/31/2014. Enum, available values: Empty, DisplayName, FirstName, LastName, Organization, LastFirstMiddle, OrgLastFirstMiddle, LastFirstMiddleOrg, LastFirstMiddle2, LastFirstMiddle3, OrgLastFirstMiddle2, OrgLastFirstMiddle3, LastFirstMiddleOrg2, LastFirstMiddleOrg3, LastFirstMiddleGen, FirstMiddleLastGen, LastFirstMiddleGen2, BestMatch, AccordingToLocale, None
        :type file_as_mapping: str
        :param free_busy_location: URL path from which a client can retrieve free/busy information for the contact as an iCal file.             
        :type free_busy_location: str
        :param gender: Enum defines gender of a person. Enum, available values: Unspecified, Female, Male
        :type gender: str
        :param given_name: Person's given name.             
        :type given_name: str
        :param government_id_number: Government id number.             
        :type government_id_number: str
        :param hobbies: Person's hobbies.             
        :type hobbies: str
        :param initials: Person's initials.             
        :type initials: str
        :param instant_messengers: Person's instant messenger addresses.             
        :type instant_messengers: List[InstantMessengerAddress]
        :param job_title: Person's job title.             
        :type job_title: str
        :param language: Language.             
        :type language: str
        :param location: Person's location.             
        :type location: str
        :param middle_name: Person's middle name.             
        :type middle_name: str
        :param nickname: Person's nickname.             
        :type nickname: str
        :param notes: Notes.             
        :type notes: str
        :param notes_format: Defines format of a text. Enum, available values: Text, Html
        :type notes_format: str
        :param office_location: Office location.             
        :type office_location: str
        :param organizational_id_number: Contains an identifier for the mail user used within the mail user's organization.             
        :type organizational_id_number: str
        :param phone_numbers: Person's phone numbers.             
        :type phone_numbers: List[PhoneNumber]
        :param photo: Person's photo.             
        :type photo: ContactPhoto
        :param physical_addresses: Person's physical addresses.             
        :type physical_addresses: List[PostalAddress]
        :param preferred_text_encoding: Encoding for all text properties.             
        :type preferred_text_encoding: str
        :param prefix: A prefix of a full name such like Mr.(mister), Dr.(doctor) and so on.             
        :type prefix: str
        :param profession: A job position of a person in a company.             
        :type profession: str
        :param suffix: A suffix of a full name such like Jr.(junior), Sr.(senior) and so on.             
        :type suffix: str
        :param surname: Person's surname.             
        :type surname: str
        :param urls: Person's urls.             
        :type urls: List[Url]
        """

        self._associated_persons = None
        self._attachments = None
        self._company_name = None
        self._computer_network_name = None
        self._customer_id = None
        self._department_name = None
        self._display_name = None
        self._email_addresses = None
        self._events = None
        self._file_as = None
        self._file_as_mapping = None
        self._free_busy_location = None
        self._gender = None
        self._given_name = None
        self._government_id_number = None
        self._hobbies = None
        self._initials = None
        self._instant_messengers = None
        self._job_title = None
        self._language = None
        self._location = None
        self._middle_name = None
        self._nickname = None
        self._notes = None
        self._notes_format = None
        self._office_location = None
        self._organizational_id_number = None
        self._phone_numbers = None
        self._photo = None
        self._physical_addresses = None
        self._preferred_text_encoding = None
        self._prefix = None
        self._profession = None
        self._suffix = None
        self._surname = None
        self._urls = None

        if associated_persons is not None:
            self.associated_persons = associated_persons
        if attachments is not None:
            self.attachments = attachments
        if company_name is not None:
            self.company_name = company_name
        if computer_network_name is not None:
            self.computer_network_name = computer_network_name
        if customer_id is not None:
            self.customer_id = customer_id
        if department_name is not None:
            self.department_name = department_name
        if display_name is not None:
            self.display_name = display_name
        if email_addresses is not None:
            self.email_addresses = email_addresses
        if events is not None:
            self.events = events
        if file_as is not None:
            self.file_as = file_as
        if file_as_mapping is not None:
            self.file_as_mapping = file_as_mapping
        if free_busy_location is not None:
            self.free_busy_location = free_busy_location
        if gender is not None:
            self.gender = gender
        if given_name is not None:
            self.given_name = given_name
        if government_id_number is not None:
            self.government_id_number = government_id_number
        if hobbies is not None:
            self.hobbies = hobbies
        if initials is not None:
            self.initials = initials
        if instant_messengers is not None:
            self.instant_messengers = instant_messengers
        if job_title is not None:
            self.job_title = job_title
        if language is not None:
            self.language = language
        if location is not None:
            self.location = location
        if middle_name is not None:
            self.middle_name = middle_name
        if nickname is not None:
            self.nickname = nickname
        if notes is not None:
            self.notes = notes
        if notes_format is not None:
            self.notes_format = notes_format
        if office_location is not None:
            self.office_location = office_location
        if organizational_id_number is not None:
            self.organizational_id_number = organizational_id_number
        if phone_numbers is not None:
            self.phone_numbers = phone_numbers
        if photo is not None:
            self.photo = photo
        if physical_addresses is not None:
            self.physical_addresses = physical_addresses
        if preferred_text_encoding is not None:
            self.preferred_text_encoding = preferred_text_encoding
        if prefix is not None:
            self.prefix = prefix
        if profession is not None:
            self.profession = profession
        if suffix is not None:
            self.suffix = suffix
        if surname is not None:
            self.surname = surname
        if urls is not None:
            self.urls = urls


    @property
    def associated_persons(self) -> List[AssociatedPerson]:
        """
        Associated persons.             

        :return: The associated_persons of this ContactDto.
        :rtype: list[AssociatedPerson]
        """
        return self._associated_persons

    @associated_persons.setter
    def associated_persons(self, associated_persons: List[AssociatedPerson]):
        """
        Associated persons.             

        :param associated_persons: The associated_persons of this ContactDto.
        :type: list[AssociatedPerson]
        """
        self._associated_persons = associated_persons

    @property
    def attachments(self) -> List[Attachment]:
        """
        Document attachments.             

        :return: The attachments of this ContactDto.
        :rtype: list[Attachment]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments: List[Attachment]):
        """
        Document attachments.             

        :param attachments: The attachments of this ContactDto.
        :type: list[Attachment]
        """
        self._attachments = attachments

    @property
    def company_name(self) -> str:
        """
        Company name.             

        :return: The company_name of this ContactDto.
        :rtype: str
        """
        return self._company_name

    @company_name.setter
    def company_name(self, company_name: str):
        """
        Company name.             

        :param company_name: The company_name of this ContactDto.
        :type: str
        """
        self._company_name = company_name

    @property
    def computer_network_name(self) -> str:
        """
        Computer network.             

        :return: The computer_network_name of this ContactDto.
        :rtype: str
        """
        return self._computer_network_name

    @computer_network_name.setter
    def computer_network_name(self, computer_network_name: str):
        """
        Computer network.             

        :param computer_network_name: The computer_network_name of this ContactDto.
        :type: str
        """
        self._computer_network_name = computer_network_name

    @property
    def customer_id(self) -> str:
        """
        Customer id.             

        :return: The customer_id of this ContactDto.
        :rtype: str
        """
        return self._customer_id

    @customer_id.setter
    def customer_id(self, customer_id: str):
        """
        Customer id.             

        :param customer_id: The customer_id of this ContactDto.
        :type: str
        """
        self._customer_id = customer_id

    @property
    def department_name(self) -> str:
        """
        Department name.             

        :return: The department_name of this ContactDto.
        :rtype: str
        """
        return self._department_name

    @department_name.setter
    def department_name(self, department_name: str):
        """
        Department name.             

        :param department_name: The department_name of this ContactDto.
        :type: str
        """
        self._department_name = department_name

    @property
    def display_name(self) -> str:
        """
        Display name.             

        :return: The display_name of this ContactDto.
        :rtype: str
        """
        return self._display_name

    @display_name.setter
    def display_name(self, display_name: str):
        """
        Display name.             

        :param display_name: The display_name of this ContactDto.
        :type: str
        """
        self._display_name = display_name

    @property
    def email_addresses(self) -> List[EmailAddress]:
        """
        Person's email addresses.             

        :return: The email_addresses of this ContactDto.
        :rtype: list[EmailAddress]
        """
        return self._email_addresses

    @email_addresses.setter
    def email_addresses(self, email_addresses: List[EmailAddress]):
        """
        Person's email addresses.             

        :param email_addresses: The email_addresses of this ContactDto.
        :type: list[EmailAddress]
        """
        self._email_addresses = email_addresses

    @property
    def events(self) -> List[CustomerEvent]:
        """
        Person's events.             

        :return: The events of this ContactDto.
        :rtype: list[CustomerEvent]
        """
        return self._events

    @events.setter
    def events(self, events: List[CustomerEvent]):
        """
        Person's events.             

        :param events: The events of this ContactDto.
        :type: list[CustomerEvent]
        """
        self._events = events

    @property
    def file_as(self) -> str:
        """
        A name used for sorting.             

        :return: The file_as of this ContactDto.
        :rtype: str
        """
        return self._file_as

    @file_as.setter
    def file_as(self, file_as: str):
        """
        A name used for sorting.             

        :param file_as: The file_as of this ContactDto.
        :type: str
        """
        self._file_as = file_as

    @property
    def file_as_mapping(self) -> str:
        """
        Specifies how to generate and recompute the value of the dispidFileAs property when other contact name properties change. Coincides MS-OXPROPS revision 16.2 from 7/31/2014. Enum, available values: Empty, DisplayName, FirstName, LastName, Organization, LastFirstMiddle, OrgLastFirstMiddle, LastFirstMiddleOrg, LastFirstMiddle2, LastFirstMiddle3, OrgLastFirstMiddle2, OrgLastFirstMiddle3, LastFirstMiddleOrg2, LastFirstMiddleOrg3, LastFirstMiddleGen, FirstMiddleLastGen, LastFirstMiddleGen2, BestMatch, AccordingToLocale, None

        :return: The file_as_mapping of this ContactDto.
        :rtype: str
        """
        return self._file_as_mapping

    @file_as_mapping.setter
    def file_as_mapping(self, file_as_mapping: str):
        """
        Specifies how to generate and recompute the value of the dispidFileAs property when other contact name properties change. Coincides MS-OXPROPS revision 16.2 from 7/31/2014. Enum, available values: Empty, DisplayName, FirstName, LastName, Organization, LastFirstMiddle, OrgLastFirstMiddle, LastFirstMiddleOrg, LastFirstMiddle2, LastFirstMiddle3, OrgLastFirstMiddle2, OrgLastFirstMiddle3, LastFirstMiddleOrg2, LastFirstMiddleOrg3, LastFirstMiddleGen, FirstMiddleLastGen, LastFirstMiddleGen2, BestMatch, AccordingToLocale, None

        :param file_as_mapping: The file_as_mapping of this ContactDto.
        :type: str
        """
        if file_as_mapping is None:
            raise ValueError("Invalid value for `file_as_mapping`, must not be `None`")
        self._file_as_mapping = file_as_mapping

    @property
    def free_busy_location(self) -> str:
        """
        URL path from which a client can retrieve free/busy information for the contact as an iCal file.             

        :return: The free_busy_location of this ContactDto.
        :rtype: str
        """
        return self._free_busy_location

    @free_busy_location.setter
    def free_busy_location(self, free_busy_location: str):
        """
        URL path from which a client can retrieve free/busy information for the contact as an iCal file.             

        :param free_busy_location: The free_busy_location of this ContactDto.
        :type: str
        """
        self._free_busy_location = free_busy_location

    @property
    def gender(self) -> str:
        """
        Enum defines gender of a person. Enum, available values: Unspecified, Female, Male

        :return: The gender of this ContactDto.
        :rtype: str
        """
        return self._gender

    @gender.setter
    def gender(self, gender: str):
        """
        Enum defines gender of a person. Enum, available values: Unspecified, Female, Male

        :param gender: The gender of this ContactDto.
        :type: str
        """
        if gender is None:
            raise ValueError("Invalid value for `gender`, must not be `None`")
        self._gender = gender

    @property
    def given_name(self) -> str:
        """
        Person's given name.             

        :return: The given_name of this ContactDto.
        :rtype: str
        """
        return self._given_name

    @given_name.setter
    def given_name(self, given_name: str):
        """
        Person's given name.             

        :param given_name: The given_name of this ContactDto.
        :type: str
        """
        self._given_name = given_name

    @property
    def government_id_number(self) -> str:
        """
        Government id number.             

        :return: The government_id_number of this ContactDto.
        :rtype: str
        """
        return self._government_id_number

    @government_id_number.setter
    def government_id_number(self, government_id_number: str):
        """
        Government id number.             

        :param government_id_number: The government_id_number of this ContactDto.
        :type: str
        """
        self._government_id_number = government_id_number

    @property
    def hobbies(self) -> str:
        """
        Person's hobbies.             

        :return: The hobbies of this ContactDto.
        :rtype: str
        """
        return self._hobbies

    @hobbies.setter
    def hobbies(self, hobbies: str):
        """
        Person's hobbies.             

        :param hobbies: The hobbies of this ContactDto.
        :type: str
        """
        self._hobbies = hobbies

    @property
    def initials(self) -> str:
        """
        Person's initials.             

        :return: The initials of this ContactDto.
        :rtype: str
        """
        return self._initials

    @initials.setter
    def initials(self, initials: str):
        """
        Person's initials.             

        :param initials: The initials of this ContactDto.
        :type: str
        """
        self._initials = initials

    @property
    def instant_messengers(self) -> List[InstantMessengerAddress]:
        """
        Person's instant messenger addresses.             

        :return: The instant_messengers of this ContactDto.
        :rtype: list[InstantMessengerAddress]
        """
        return self._instant_messengers

    @instant_messengers.setter
    def instant_messengers(self, instant_messengers: List[InstantMessengerAddress]):
        """
        Person's instant messenger addresses.             

        :param instant_messengers: The instant_messengers of this ContactDto.
        :type: list[InstantMessengerAddress]
        """
        self._instant_messengers = instant_messengers

    @property
    def job_title(self) -> str:
        """
        Person's job title.             

        :return: The job_title of this ContactDto.
        :rtype: str
        """
        return self._job_title

    @job_title.setter
    def job_title(self, job_title: str):
        """
        Person's job title.             

        :param job_title: The job_title of this ContactDto.
        :type: str
        """
        self._job_title = job_title

    @property
    def language(self) -> str:
        """
        Language.             

        :return: The language of this ContactDto.
        :rtype: str
        """
        return self._language

    @language.setter
    def language(self, language: str):
        """
        Language.             

        :param language: The language of this ContactDto.
        :type: str
        """
        self._language = language

    @property
    def location(self) -> str:
        """
        Person's location.             

        :return: The location of this ContactDto.
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location: str):
        """
        Person's location.             

        :param location: The location of this ContactDto.
        :type: str
        """
        self._location = location

    @property
    def middle_name(self) -> str:
        """
        Person's middle name.             

        :return: The middle_name of this ContactDto.
        :rtype: str
        """
        return self._middle_name

    @middle_name.setter
    def middle_name(self, middle_name: str):
        """
        Person's middle name.             

        :param middle_name: The middle_name of this ContactDto.
        :type: str
        """
        self._middle_name = middle_name

    @property
    def nickname(self) -> str:
        """
        Person's nickname.             

        :return: The nickname of this ContactDto.
        :rtype: str
        """
        return self._nickname

    @nickname.setter
    def nickname(self, nickname: str):
        """
        Person's nickname.             

        :param nickname: The nickname of this ContactDto.
        :type: str
        """
        self._nickname = nickname

    @property
    def notes(self) -> str:
        """
        Notes.             

        :return: The notes of this ContactDto.
        :rtype: str
        """
        return self._notes

    @notes.setter
    def notes(self, notes: str):
        """
        Notes.             

        :param notes: The notes of this ContactDto.
        :type: str
        """
        self._notes = notes

    @property
    def notes_format(self) -> str:
        """
        Defines format of a text. Enum, available values: Text, Html

        :return: The notes_format of this ContactDto.
        :rtype: str
        """
        return self._notes_format

    @notes_format.setter
    def notes_format(self, notes_format: str):
        """
        Defines format of a text. Enum, available values: Text, Html

        :param notes_format: The notes_format of this ContactDto.
        :type: str
        """
        if notes_format is None:
            raise ValueError("Invalid value for `notes_format`, must not be `None`")
        self._notes_format = notes_format

    @property
    def office_location(self) -> str:
        """
        Office location.             

        :return: The office_location of this ContactDto.
        :rtype: str
        """
        return self._office_location

    @office_location.setter
    def office_location(self, office_location: str):
        """
        Office location.             

        :param office_location: The office_location of this ContactDto.
        :type: str
        """
        self._office_location = office_location

    @property
    def organizational_id_number(self) -> str:
        """
        Contains an identifier for the mail user used within the mail user's organization.             

        :return: The organizational_id_number of this ContactDto.
        :rtype: str
        """
        return self._organizational_id_number

    @organizational_id_number.setter
    def organizational_id_number(self, organizational_id_number: str):
        """
        Contains an identifier for the mail user used within the mail user's organization.             

        :param organizational_id_number: The organizational_id_number of this ContactDto.
        :type: str
        """
        self._organizational_id_number = organizational_id_number

    @property
    def phone_numbers(self) -> List[PhoneNumber]:
        """
        Person's phone numbers.             

        :return: The phone_numbers of this ContactDto.
        :rtype: list[PhoneNumber]
        """
        return self._phone_numbers

    @phone_numbers.setter
    def phone_numbers(self, phone_numbers: List[PhoneNumber]):
        """
        Person's phone numbers.             

        :param phone_numbers: The phone_numbers of this ContactDto.
        :type: list[PhoneNumber]
        """
        self._phone_numbers = phone_numbers

    @property
    def photo(self) -> ContactPhoto:
        """
        Person's photo.             

        :return: The photo of this ContactDto.
        :rtype: ContactPhoto
        """
        return self._photo

    @photo.setter
    def photo(self, photo: ContactPhoto):
        """
        Person's photo.             

        :param photo: The photo of this ContactDto.
        :type: ContactPhoto
        """
        self._photo = photo

    @property
    def physical_addresses(self) -> List[PostalAddress]:
        """
        Person's physical addresses.             

        :return: The physical_addresses of this ContactDto.
        :rtype: list[PostalAddress]
        """
        return self._physical_addresses

    @physical_addresses.setter
    def physical_addresses(self, physical_addresses: List[PostalAddress]):
        """
        Person's physical addresses.             

        :param physical_addresses: The physical_addresses of this ContactDto.
        :type: list[PostalAddress]
        """
        self._physical_addresses = physical_addresses

    @property
    def preferred_text_encoding(self) -> str:
        """
        Encoding for all text properties.             

        :return: The preferred_text_encoding of this ContactDto.
        :rtype: str
        """
        return self._preferred_text_encoding

    @preferred_text_encoding.setter
    def preferred_text_encoding(self, preferred_text_encoding: str):
        """
        Encoding for all text properties.             

        :param preferred_text_encoding: The preferred_text_encoding of this ContactDto.
        :type: str
        """
        self._preferred_text_encoding = preferred_text_encoding

    @property
    def prefix(self) -> str:
        """
        A prefix of a full name such like Mr.(mister), Dr.(doctor) and so on.             

        :return: The prefix of this ContactDto.
        :rtype: str
        """
        return self._prefix

    @prefix.setter
    def prefix(self, prefix: str):
        """
        A prefix of a full name such like Mr.(mister), Dr.(doctor) and so on.             

        :param prefix: The prefix of this ContactDto.
        :type: str
        """
        self._prefix = prefix

    @property
    def profession(self) -> str:
        """
        A job position of a person in a company.             

        :return: The profession of this ContactDto.
        :rtype: str
        """
        return self._profession

    @profession.setter
    def profession(self, profession: str):
        """
        A job position of a person in a company.             

        :param profession: The profession of this ContactDto.
        :type: str
        """
        self._profession = profession

    @property
    def suffix(self) -> str:
        """
        A suffix of a full name such like Jr.(junior), Sr.(senior) and so on.             

        :return: The suffix of this ContactDto.
        :rtype: str
        """
        return self._suffix

    @suffix.setter
    def suffix(self, suffix: str):
        """
        A suffix of a full name such like Jr.(junior), Sr.(senior) and so on.             

        :param suffix: The suffix of this ContactDto.
        :type: str
        """
        self._suffix = suffix

    @property
    def surname(self) -> str:
        """
        Person's surname.             

        :return: The surname of this ContactDto.
        :rtype: str
        """
        return self._surname

    @surname.setter
    def surname(self, surname: str):
        """
        Person's surname.             

        :param surname: The surname of this ContactDto.
        :type: str
        """
        self._surname = surname

    @property
    def urls(self) -> List[Url]:
        """
        Person's urls.             

        :return: The urls of this ContactDto.
        :rtype: list[Url]
        """
        return self._urls

    @urls.setter
    def urls(self, urls: List[Url]):
        """
        Person's urls.             

        :param urls: The urls of this ContactDto.
        :type: list[Url]
        """
        self._urls = urls

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ContactDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="rest.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

from __future__ import absolute_import

import datetime
import inspect
import io
import json
import logging
import mimetypes
import os
import re
import ssl
import tempfile

import certifi
import six
# python 2 and python 3 compatibility library
from six.moves.urllib.parse import urlencode

import AsposeEmailCloudSdk
from AsposeEmailCloudSdk.models.error import Error

try:
    import urllib3
except ImportError:
    raise ImportError('Swagger python client requires urllib3.')


LOGGER = logging.getLogger(__name__)


class RESTResponse(io.IOBase):

    def __init__(self, resp):
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = resp.data

    def getheaders(self):
        """Returns a dictionary of the response headers."""
        return self.urllib3_response.getheaders()

    def getheader(self, name, default=None):
        """Returns a given response header."""
        return self.urllib3_response.getheader(name, default)


class RESTClientObject(object):

    def __init__(self, configuration, pools_size=4, maxsize=None):
        # pylint: disable=line-too-long
        # urllib3.PoolManager will pass all kw parameters to connectionpool
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/poolmanager.py#L75
        # https://github.com/shazow/urllib3/blob/f9409436f83aeb79fbaf090181cd81b784f1b8ce/urllib3/connectionpool.py#L680
        # maxsize is the number of requests to host that are allowed in parallel
        # Custom SSL certificates and client certificates: http://urllib3.readthedocs.io/en/latest/advanced-usage.html
        # pylint: enable=line-too-long

        # cert_reqs
        if configuration.verify_ssl:
            cert_reqs = ssl.CERT_REQUIRED
        else:
            cert_reqs = ssl.CERT_NONE

        # ca_certs
        if configuration.ssl_ca_cert:
            ca_certs = configuration.ssl_ca_cert# For `GET`, `HEAD`
        else:
            # if not set certificate file, use Mozilla's root certificates.
            ca_certs = certifi.where()

        addition_pool_args = {}
        if configuration.assert_hostname is not None:
            addition_pool_args['assert_hostname'] = configuration.assert_hostname

        if maxsize is None:
            if configuration.connection_pool_maxsize is not None:
                maxsize = configuration.connection_pool_maxsize
            else:
                maxsize = 4

        if configuration.temp_folder_path:
            self.temp_folder_path = configuration.temp_folder_path
        else:
            self.temp_folder_path = None

        # https pool manager
        if configuration.proxy:
            self.pool_manager = urllib3.ProxyManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=cert_reqs,
                ca_certs=ca_certs,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                proxy_url=configuration.proxy,
                **addition_pool_args
            )
        else:
            self.pool_manager = urllib3.PoolManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=cert_reqs,
                ca_certs=ca_certs,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                **addition_pool_args
            )

    def request(self, method, url, query_params=None, headers=None,
                body=None, post_params=None, _preload_content=True,
                _request_timeout=None):
        """Perform requests.

        :param method: http request method
        :param url: http request url
        :param query_params: query parameters in the url
        :param headers: http request headers
        :param body: request json body, for `application/json`
        :param post_params: request post parameters,
                            `application/x-www-form-urlencoded`
                            and `multipart/form-data`
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        """
        method = method.upper()
        assert method in ['GET', 'HEAD', 'DELETE', 'POST', 'PUT',
                          'PATCH', 'OPTIONS']

        if post_params and body:
            raise ValueError(
                "body parameter cannot be used with post_params parameter."
            )

        post_params = post_params or {}
        headers = headers or {}

        timeout = None
        if _request_timeout:
            if isinstance(_request_timeout, (int, ) if six.PY3 else (
                    int, long)):  # pylint: disable=undefined-variable
                timeout = urllib3.Timeout(total=_request_timeout)
            elif (isinstance(_request_timeout, tuple) and
                  len(_request_timeout) == 2):
                timeout = urllib3.Timeout(
                    connect=_request_timeout[0], read=_request_timeout[1])

        if 'Content-Type' not in headers:
            headers['Content-Type'] = 'application/json'

        try:
            # For `POST`, `PUT`, `PATCH`, `OPTIONS`, `DELETE`
            if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:
                if query_params:
                    url += '?' + urlencode(query_params)
                if not six.PY3:
                    url = url.encode('utf8')
                if re.search('json', headers['Content-Type'], re.IGNORECASE):
                    request_body = ''
                    if body is not None:
                        request_body = json.dumps(body)
                    req = self.pool_manager.request(
                        method, url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                elif headers['Content-Type'] == 'application/x-www-form-urlencoded':
                    req = self.pool_manager.request(
                        method, url,
                        fields=post_params,
                        encode_multipart=False,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                elif headers['Content-Type'] == 'multipart/form-data':
                    # must del headers['Content-Type'], or the correct
                    # Content-Type which generated by urllib3 will be
                    # overwritten.
                    del headers['Content-Type']
                    req = self.pool_manager.request(
                        method, url,
                        fields=post_params,
                        encode_multipart=True,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                # Pass a `string` parameter directly in the body to support
                # other content types than Json when `body` argument is
                # provided in serialized form
                elif isinstance(body, str):
                    request_body = body
                    req = self.pool_manager.request(
                        method, url,
                        body=request_body,
                        preload_content=_preload_content,
                        timeout=timeout,
                        headers=headers)
                else:
                    # Cannot generate the request from given parameters
                    msg = """Cannot prepare a request message for provided
                             arguments. Please check that your arguments match
                             declared content type."""
                    raise ApiException(message=msg, code=0)
            # For `GET`, `HEAD`
            else:
                request_body = ''
                if body is not None:
                    request_body = json.dumps(body)
                req = self.pool_manager.request(
                    method, url,
                    body=request_body,
                    fields=query_params,
                    preload_content=_preload_content,
                    timeout=timeout,
                    headers=headers)
        except urllib3.exceptions.SSLError as ex:
            msg = "{0}\n{1}".format(type(ex).__name__, str(ex))
            raise ApiException(message=msg, code=0)

        if _preload_content:
            req = RESTResponse(req)

            # log response body
            LOGGER.debug("response body: %s", req.data)

        if not 200 <= req.status <= 299:
            error = None
            try:
                if six.PY3:
                    req.data = req.data.decode('utf8')
                req.data = json.loads(req.data)["error"]
                error = ObjectHelper.deserialize(
                    req, 'Error', self.temp_folder_path)
            finally:
                raise ApiException(
                    '{0} Error connecting to the API {1}; Message: {2}'
                    .format(req.status, url, req.reason), req.status, error)

        return req

    def get(self, url, headers=None, query_params=None, _preload_content=True,
            _request_timeout=None):
        return self.request("GET", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def head(self, url, headers=None, query_params=None, _preload_content=True,
             _request_timeout=None):
        return self.request("HEAD", url,
                            headers=headers,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            query_params=query_params)

    def options(self, url, headers=None, query_params=None, post_params=None,
                body=None, _preload_content=True, _request_timeout=None):
        return self.request("OPTIONS", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def delete(self, url, headers=None, query_params=None, body=None,
               _preload_content=True, _request_timeout=None):
        return self.request("DELETE", url,
                            headers=headers,
                            query_params=query_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def post(self, url, headers=None, query_params=None, post_params=None,
             body=None, _preload_content=True, _request_timeout=None):
        return self.request("POST", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def put(self, url, headers=None, query_params=None, post_params=None,
            body=None, _preload_content=True, _request_timeout=None):
        return self.request("PUT", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)

    def patch(self, url, headers=None, query_params=None, post_params=None,
              body=None, _preload_content=True, _request_timeout=None):
        return self.request("PATCH", url,
                            headers=headers,
                            query_params=query_params,
                            post_params=post_params,
                            _preload_content=_preload_content,
                            _request_timeout=_request_timeout,
                            body=body)


class ApiException(Exception):
    """
    API Exception
    """

    def __init__(self, message, code, error=None):
        """
        Initializes a new instance of the ApiException class
        :param message: Error message
        :param code: Error code
        :param error: Error
        """
        self.message = message
        self.code = code
        if error:
            self.error = error


class ObjectHelper:
    PRIMITIVE_TYPES = (float, bool, bytes, six.text_type) + six.integer_types
    NATIVE_TYPES_MAPPING = {
        'int': int,
        'long': int if six.PY3 else long,  # pylint: disable=undefined-variable
        'float': float,
        'str': str,
        'bool': bool,
        'date': datetime.date,
        'datetime': datetime.datetime,
        'object': object,
    }

    @staticmethod
    def sanitize_for_serialization(obj):
        """Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is swagger model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
        if obj is None:
            return None
        elif isinstance(obj, ObjectHelper.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [ObjectHelper.sanitize_for_serialization(sub_obj)
                    for sub_obj in obj]
        elif isinstance(obj, tuple):
            return tuple(ObjectHelper.sanitize_for_serialization(sub_obj)
                         for sub_obj in obj)
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()

        if isinstance(obj, dict):
            obj_dict = obj
        else:
            # Convert model obj to dict except
            # attributes `swagger_types`, `attribute_map`
            # and attributes which value is not None.
            # Convert attribute name to json key in
            # model definition for request.
            obj_dict = {obj.attribute_map[attr]: getattr(obj, attr)
                        for attr, _ in six.iteritems(obj.swagger_types)
                        if getattr(obj, attr) is not None}

        return {key: ObjectHelper.sanitize_for_serialization(val)
                for key, val in six.iteritems(obj_dict)}

    @staticmethod
    def deserialize(response, response_type, temp_folder_path=None):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.
        :param temp_folder_path: path to local temp folder.

        :return: deserialized object.
        """
        # check if data from the response object is already processed
        if isinstance(response.data, dict):
            return ObjectHelper.__deserialize(response.data, response_type)

        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == "file":
            return ObjectHelper.__deserialize_file(response, temp_folder_path)

        # fetch data from response object
        if six.PY3:
            response.data = response.data.decode('utf8')
        try:
            data = json.loads(response.data)
        except ValueError:
            data = response.data

        return ObjectHelper.__deserialize(data, response_type)

    @staticmethod
    def __deserialize(data, klass):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
        if data is None:
            return None

        if isinstance(klass, str):
            if klass.startswith('list['):
                sub_kls = re.match(r'list\[(.*)\]', klass).group(1)
                return [ObjectHelper.__deserialize(sub_data, sub_kls)
                        for sub_data in data]

            if klass.startswith('dict('):
                sub_kls = re.match(r'dict\(([^,]*), (.*)\)', klass).group(2)
                return {k: ObjectHelper.__deserialize(v, sub_kls)
                        for k, v in six.iteritems(data)}

            # convert str to class
            if klass in ObjectHelper.NATIVE_TYPES_MAPPING:
                klass = ObjectHelper.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = getattr(AsposeEmailCloudSdk.models, klass)

        if klass in ObjectHelper.PRIMITIVE_TYPES:
            return ObjectHelper.__deserialize_primitive(data, klass)
        elif klass == object:
            return ObjectHelper.__deserialize_object(data)
        elif klass == datetime.date:
            return ObjectHelper.__deserialize_date(data)
        elif klass == datetime.datetime:
            return ObjectHelper.__deserialize_datatime(data)

        return ObjectHelper.__deserialize_model(data, klass)

    @staticmethod
    def parameters_to_tuples(params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: Parameters as list of tuples, collections formatted
        """
        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in six.iteritems(params) if isinstance(params,
                                                        dict) else params:
            if k in collection_formats:
                collection_format = collection_formats[k]
                if collection_format == 'multi':
                    new_params.extend((k, value) for value in v)
                else:
                    if collection_format == 'ssv':
                        delimiter = ' '
                    elif collection_format == 'tsv':
                        delimiter = '\t'
                    elif collection_format == 'pipes':
                        delimiter = '|'
                    else:  # csv is the default
                        delimiter = ','
                    new_params.append(
                        (k, delimiter.join(str(value) for value in v)))
            else:
                new_params.append((k, v))
        return new_params

    @staticmethod
    def prepare_post_parameters(post_params=None, files=None):
        """Builds form parameters.

        :param post_params: Normal form parameters.
        :param files: File parameters.
        :return: Form parameters with files.
        """
        params = []

        if post_params:
            params = post_params

        if files:
            for k, v in files:
                if not v:
                    continue
                file_names = v if isinstance(v, list) else [v]
                for n in file_names:
                    with open(n, 'rb') as f:
                        filename = os.path.basename(f.name)
                        filedata = f.read()
                        mimetype = (mimetypes.guess_type(filename)[0] or
                                    'application/octet-stream')
                        params.append(
                            tuple([k, tuple([filename, filedata, mimetype])]))

        return params

    @staticmethod
    def __deserialize_file(response, temp_folder_path):
        """Deserializes body to file

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :param temp_folder_path: Path to local temp folder.
        :return: file path.
        """
        fd, path = tempfile.mkstemp(dir=temp_folder_path)
        os.close(fd)
        os.remove(path)

        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?',
                                 content_disposition).group(1)
            filename = filename.replace('/', '_')
            path = os.path.join(os.path.dirname(path), filename)

        with open(path, "wb") as f:
            f.write(response.data)

        return path

    @staticmethod
    def __deserialize_primitive(data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            if not six.PY3:
                return data.encode('utf8')
            return six.u(data)
        except TypeError:
            return data

    @staticmethod
    def __deserialize_object(value):
        """Return a original value.

        :return: object.
        """
        return value

    @staticmethod
    def __deserialize_date(string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            from dateutil.parser import parse
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise ApiException(
                "Failed to parse `{0}` as date object".format(string),
                0
            )

    @staticmethod
    def __deserialize_datatime(string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            from dateutil.parser import parse
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise ApiException(
                "Failed to parse `{0}` as datetime object"
                    .format(string), 0
            )

    @staticmethod
    def __deserialize_model(data, klass):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """

        if (not klass.swagger_types and
                not hasattr(klass, 'get_real_child_model')):
            return data

        #try get derived class by type field
        type_attribute = 'discriminator'
        if (hasattr(klass, type_attribute) and type_attribute in data):
            type_name = data[type_attribute]
            sub_klass = getattr(AsposeEmailCloudSdk.models, type_name, None)
            if (sub_klass is not None):
                klass = sub_klass

        kwargs = {}
        if klass.swagger_types is not None:
            for attr, attr_type in six.iteritems(klass.swagger_types):
                if (data is not None and
                        not attr == type_attribute and
                        klass.attribute_map[attr][0].lower() +
                        klass.attribute_map[attr][1:] in data and
                        isinstance(data, (list, dict))):
                    value = data[klass.attribute_map[attr][0].lower() +
                                 klass.attribute_map[attr][1:]]
                    kwargs[attr] = ObjectHelper.__deserialize(value, attr_type)

        instance = klass(**kwargs)

        if (isinstance(instance, dict) and
                klass.swagger_types is not None and
                isinstance(data, dict)):
            for key, value in data.items():
                if key not in klass.swagger_types:
                    instance[key] = value

        base = ObjectHelper.__get_base_class(instance)
        if base and base.swagger_types is not None:
            for attr, attr_type in six.iteritems(base.swagger_types):
                if (data is not None and
                        base.attribute_map[attr][0].lower() +
                        base.attribute_map[attr][1:] in data and
                        isinstance(data, (list, dict))):
                    value = data[base.attribute_map[attr][0].lower() +
                                 base.attribute_map[attr][1:]]
                    setattr(instance, attr,
                            ObjectHelper.__deserialize(value, attr_type))
        return instance

    @staticmethod
    def __get_base_class(klass):
        return next((x for x in inspect.getmro(type(klass)) if
                     x != klass.__class__ and x != object), None)



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="storage_api.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

from __future__ import absolute_import

from AsposeEmailCloudSdk.api.api_base import ApiBase
from AsposeEmailCloudSdk.models import *


class StorageApi(ApiBase):
    """
    Aspose.Email Cloud API. StorageApi operations.

    """

    def __init__(self, api_client):
        super(StorageApi, self).__init__(api_client)

    def get_disc_usage(self, request: GetDiscUsageRequest) -> DiscUsage:
        """Get disc usage


        :param request: GetDiscUsageRequest object with parameters
        :type request: GetDiscUsageRequest
        :return: DiscUsage
        """

        collection_formats = {}
        path = '/email/storage/disc'
        path_params = {}

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('storageName') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.storage_name if request.storage_name is not None else '')
        else:
            if request.storage_name is not None:
                query_params.append((self._lowercase_first_letter('storageName'), request.storage_name))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'DiscUsage')

    def get_file_versions(self, request: GetFileVersionsRequest) -> FileVersions:
        """Get file versions


        :param request: GetFileVersionsRequest object with parameters
        :type request: GetFileVersionsRequest
        :return: FileVersions
        """
        # verify the required parameter 'path' is set
        if request.path is None:
            raise ValueError("Missing the required parameter `path` when calling `get_file_versions`")

        collection_formats = {}
        path = '/email/storage/version/{path}'
        path_params = {}
        if request.path is not None:
            path_params[self._lowercase_first_letter('path')] = request.path

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('storageName') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.storage_name if request.storage_name is not None else '')
        else:
            if request.storage_name is not None:
                query_params.append((self._lowercase_first_letter('storageName'), request.storage_name))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'FileVersions')

    def object_exists(self, request: ObjectExistsRequest) -> ObjectExist:
        """Check if file or folder exists


        :param request: ObjectExistsRequest object with parameters
        :type request: ObjectExistsRequest
        :return: ObjectExist
        """
        # verify the required parameter 'path' is set
        if request.path is None:
            raise ValueError("Missing the required parameter `path` when calling `object_exists`")

        collection_formats = {}
        path = '/email/storage/exist/{path}'
        path_params = {}
        if request.path is not None:
            path_params[self._lowercase_first_letter('path')] = request.path

        query_params = []
        path_parameter = '{' + self._lowercase_first_letter('storageName') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.storage_name if request.storage_name is not None else '')
        else:
            if request.storage_name is not None:
                query_params.append((self._lowercase_first_letter('storageName'), request.storage_name))
        path_parameter = '{' + self._lowercase_first_letter('versionId') + '}'
        if path_parameter in path:
            path = path.replace(path_parameter, request.version_id if request.version_id is not None else '')
        else:
            if request.version_id is not None:
                query_params.append((self._lowercase_first_letter('versionId'), request.version_id))

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'ObjectExist')

    def exists(self, request: StorageExistsRequest) -> StorageExist:
        """Check if storage exists


        :param request: StorageExistsRequest object with parameters
        :type request: StorageExistsRequest
        :return: StorageExist
        """
        # verify the required parameter 'storage_name' is set
        if request.storage_name is None:
            raise ValueError("Missing the required parameter `storage_name` when calling `exists`")

        collection_formats = {}
        path = '/email/storage/{storageName}/exist'
        path_params = {}
        if request.storage_name is not None:
            path_params[self._lowercase_first_letter('storageName')] = request.storage_name

        query_params = []

        form_params = []
        local_var_files = []

        header_params = {}
        # HTTP header `Accept`
        header_params['Accept'] = self._select_header_accept(
            ['application/json'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self._select_header_content_type(
            ['application/json'])

        # Authentication setting
        auth_settings = ['JWT']

        http_request_object = HttpRequest(path, path_params, query_params, header_params, form_params, None, local_var_files,
                                          collection_formats, auth_settings)

        return self._make_request(http_request_object, 'GET', 'StorageExist')




#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="MapiMessageItemBaseDto.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

import pprint
import re
import six
from typing import List, Set, Dict, Tuple, Optional
from datetime import datetime

from AsposeEmailCloudSdk.models.mapi_attachment_dto import MapiAttachmentDto
from AsposeEmailCloudSdk.models.mapi_property_dto import MapiPropertyDto
from AsposeEmailCloudSdk.models.mapi_recipient_dto import MapiRecipientDto


class MapiMessageItemBaseDto(object):
    """Base Dto for MapiMessage, MapiCalendar or MapiContact             
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'attachments': 'list[MapiAttachmentDto]',
        'billing': 'str',
        'body': 'str',
        'body_html': 'str',
        'body_rtf': 'str',
        'body_type': 'str',
        'categories': 'list[str]',
        'companies': 'list[str]',
        'item_id': 'str',
        'message_class': 'str',
        'mileage': 'str',
        'recipients': 'list[MapiRecipientDto]',
        'sensitivity': 'str',
        'subject': 'str',
        'subject_prefix': 'str',
        'properties': 'list[MapiPropertyDto]',
        'discriminator': 'str'
    }

    attribute_map = {
        'attachments': 'attachments',
        'billing': 'billing',
        'body': 'body',
        'body_html': 'bodyHtml',
        'body_rtf': 'bodyRtf',
        'body_type': 'bodyType',
        'categories': 'categories',
        'companies': 'companies',
        'item_id': 'itemId',
        'message_class': 'messageClass',
        'mileage': 'mileage',
        'recipients': 'recipients',
        'sensitivity': 'sensitivity',
        'subject': 'subject',
        'subject_prefix': 'subjectPrefix',
        'properties': 'properties',
        'discriminator': 'discriminator'
    }

    def __init__(self, attachments: List[MapiAttachmentDto] = None, billing: str = None, body: str = None, body_html: str = None, body_rtf: str = None, body_type: str = None, categories: List[str] = None, companies: List[str] = None, item_id: str = None, message_class: str = None, mileage: str = None, recipients: List[MapiRecipientDto] = None, sensitivity: str = None, subject: str = None, subject_prefix: str = None, properties: List[MapiPropertyDto] = None):
        """
        Base Dto for MapiMessage, MapiCalendar or MapiContact             
        :param attachments: Message item attachments.             
        :type attachments: List[MapiAttachmentDto]
        :param billing: Billing information associated with an item.             
        :type billing: str
        :param body: Message text.             
        :type body: str
        :param body_html: Gets the BodyRtf of the message converted to HTML, if present, otherwise an empty string.             
        :type body_html: str
        :param body_rtf: RTF formatted message text.             
        :type body_rtf: str
        :param body_type: The content type of message body. Enum, available values: PlainText, Html, Rtf
        :type body_type: str
        :param categories: Contains keywords or categories for the message object.             
        :type categories: List[str]
        :param companies: Contains the names of the companies that are associated with an item.             
        :type companies: List[str]
        :param item_id: The item id, uses with a server.             
        :type item_id: str
        :param message_class: Case-sensitive string that identifies the sender-defined message class, such as IPM.Note. The message class specifies the type, purpose, or content of the message.             
        :type message_class: str
        :param mileage: Contains the mileage information that is associated with an item.             
        :type mileage: str
        :param recipients: Recipients of the message.             
        :type recipients: List[MapiRecipientDto]
        :param sensitivity: Contains values that indicate the message sensitivity. Enum, available values: None, Personal, Private, CompanyConfidential
        :type sensitivity: str
        :param subject: Subject of the message.             
        :type subject: str
        :param subject_prefix: Subject prefix that typically indicates some action on a message, such as \"FW: \" for forwarding.             
        :type subject_prefix: str
        :param properties: List of MAPI properties             
        :type properties: List[MapiPropertyDto]
        """

        self._attachments = None
        self._billing = None
        self._body = None
        self._body_html = None
        self._body_rtf = None
        self._body_type = None
        self._categories = None
        self._companies = None
        self._item_id = None
        self._message_class = None
        self._mileage = None
        self._recipients = None
        self._sensitivity = None
        self._subject = None
        self._subject_prefix = None
        self._properties = None

        if attachments is not None:
            self.attachments = attachments
        if billing is not None:
            self.billing = billing
        if body is not None:
            self.body = body
        if body_html is not None:
            self.body_html = body_html
        if body_rtf is not None:
            self.body_rtf = body_rtf
        if body_type is not None:
            self.body_type = body_type
        if categories is not None:
            self.categories = categories
        if companies is not None:
            self.companies = companies
        if item_id is not None:
            self.item_id = item_id
        if message_class is not None:
            self.message_class = message_class
        if mileage is not None:
            self.mileage = mileage
        if recipients is not None:
            self.recipients = recipients
        if sensitivity is not None:
            self.sensitivity = sensitivity
        if subject is not None:
            self.subject = subject
        if subject_prefix is not None:
            self.subject_prefix = subject_prefix
        if properties is not None:
            self.properties = properties


    @property
    def attachments(self) -> List[MapiAttachmentDto]:
        """
        Message item attachments.             

        :return: The attachments of this MapiMessageItemBaseDto.
        :rtype: list[MapiAttachmentDto]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments: List[MapiAttachmentDto]):
        """
        Message item attachments.             

        :param attachments: The attachments of this MapiMessageItemBaseDto.
        :type: list[MapiAttachmentDto]
        """
        self._attachments = attachments

    @property
    def billing(self) -> str:
        """
        Billing information associated with an item.             

        :return: The billing of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._billing

    @billing.setter
    def billing(self, billing: str):
        """
        Billing information associated with an item.             

        :param billing: The billing of this MapiMessageItemBaseDto.
        :type: str
        """
        self._billing = billing

    @property
    def body(self) -> str:
        """
        Message text.             

        :return: The body of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._body

    @body.setter
    def body(self, body: str):
        """
        Message text.             

        :param body: The body of this MapiMessageItemBaseDto.
        :type: str
        """
        self._body = body

    @property
    def body_html(self) -> str:
        """
        Gets the BodyRtf of the message converted to HTML, if present, otherwise an empty string.             

        :return: The body_html of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._body_html

    @body_html.setter
    def body_html(self, body_html: str):
        """
        Gets the BodyRtf of the message converted to HTML, if present, otherwise an empty string.             

        :param body_html: The body_html of this MapiMessageItemBaseDto.
        :type: str
        """
        self._body_html = body_html

    @property
    def body_rtf(self) -> str:
        """
        RTF formatted message text.             

        :return: The body_rtf of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._body_rtf

    @body_rtf.setter
    def body_rtf(self, body_rtf: str):
        """
        RTF formatted message text.             

        :param body_rtf: The body_rtf of this MapiMessageItemBaseDto.
        :type: str
        """
        self._body_rtf = body_rtf

    @property
    def body_type(self) -> str:
        """
        The content type of message body. Enum, available values: PlainText, Html, Rtf

        :return: The body_type of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._body_type

    @body_type.setter
    def body_type(self, body_type: str):
        """
        The content type of message body. Enum, available values: PlainText, Html, Rtf

        :param body_type: The body_type of this MapiMessageItemBaseDto.
        :type: str
        """
        if body_type is None:
            raise ValueError("Invalid value for `body_type`, must not be `None`")
        self._body_type = body_type

    @property
    def categories(self) -> List[str]:
        """
        Contains keywords or categories for the message object.             

        :return: The categories of this MapiMessageItemBaseDto.
        :rtype: list[str]
        """
        return self._categories

    @categories.setter
    def categories(self, categories: List[str]):
        """
        Contains keywords or categories for the message object.             

        :param categories: The categories of this MapiMessageItemBaseDto.
        :type: list[str]
        """
        self._categories = categories

    @property
    def companies(self) -> List[str]:
        """
        Contains the names of the companies that are associated with an item.             

        :return: The companies of this MapiMessageItemBaseDto.
        :rtype: list[str]
        """
        return self._companies

    @companies.setter
    def companies(self, companies: List[str]):
        """
        Contains the names of the companies that are associated with an item.             

        :param companies: The companies of this MapiMessageItemBaseDto.
        :type: list[str]
        """
        self._companies = companies

    @property
    def item_id(self) -> str:
        """
        The item id, uses with a server.             

        :return: The item_id of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._item_id

    @item_id.setter
    def item_id(self, item_id: str):
        """
        The item id, uses with a server.             

        :param item_id: The item_id of this MapiMessageItemBaseDto.
        :type: str
        """
        self._item_id = item_id

    @property
    def message_class(self) -> str:
        """
        Case-sensitive string that identifies the sender-defined message class, such as IPM.Note. The message class specifies the type, purpose, or content of the message.             

        :return: The message_class of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._message_class

    @message_class.setter
    def message_class(self, message_class: str):
        """
        Case-sensitive string that identifies the sender-defined message class, such as IPM.Note. The message class specifies the type, purpose, or content of the message.             

        :param message_class: The message_class of this MapiMessageItemBaseDto.
        :type: str
        """
        self._message_class = message_class

    @property
    def mileage(self) -> str:
        """
        Contains the mileage information that is associated with an item.             

        :return: The mileage of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._mileage

    @mileage.setter
    def mileage(self, mileage: str):
        """
        Contains the mileage information that is associated with an item.             

        :param mileage: The mileage of this MapiMessageItemBaseDto.
        :type: str
        """
        self._mileage = mileage

    @property
    def recipients(self) -> List[MapiRecipientDto]:
        """
        Recipients of the message.             

        :return: The recipients of this MapiMessageItemBaseDto.
        :rtype: list[MapiRecipientDto]
        """
        return self._recipients

    @recipients.setter
    def recipients(self, recipients: List[MapiRecipientDto]):
        """
        Recipients of the message.             

        :param recipients: The recipients of this MapiMessageItemBaseDto.
        :type: list[MapiRecipientDto]
        """
        self._recipients = recipients

    @property
    def sensitivity(self) -> str:
        """
        Contains values that indicate the message sensitivity. Enum, available values: None, Personal, Private, CompanyConfidential

        :return: The sensitivity of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._sensitivity

    @sensitivity.setter
    def sensitivity(self, sensitivity: str):
        """
        Contains values that indicate the message sensitivity. Enum, available values: None, Personal, Private, CompanyConfidential

        :param sensitivity: The sensitivity of this MapiMessageItemBaseDto.
        :type: str
        """
        if sensitivity is None:
            raise ValueError("Invalid value for `sensitivity`, must not be `None`")
        self._sensitivity = sensitivity

    @property
    def subject(self) -> str:
        """
        Subject of the message.             

        :return: The subject of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._subject

    @subject.setter
    def subject(self, subject: str):
        """
        Subject of the message.             

        :param subject: The subject of this MapiMessageItemBaseDto.
        :type: str
        """
        self._subject = subject

    @property
    def subject_prefix(self) -> str:
        """
        Subject prefix that typically indicates some action on a message, such as \"FW: \" for forwarding.             

        :return: The subject_prefix of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self._subject_prefix

    @subject_prefix.setter
    def subject_prefix(self, subject_prefix: str):
        """
        Subject prefix that typically indicates some action on a message, such as \"FW: \" for forwarding.             

        :param subject_prefix: The subject_prefix of this MapiMessageItemBaseDto.
        :type: str
        """
        self._subject_prefix = subject_prefix

    @property
    def properties(self) -> List[MapiPropertyDto]:
        """
        List of MAPI properties             

        :return: The properties of this MapiMessageItemBaseDto.
        :rtype: list[MapiPropertyDto]
        """
        return self._properties

    @properties.setter
    def properties(self, properties: List[MapiPropertyDto]):
        """
        List of MAPI properties             

        :param properties: The properties of this MapiMessageItemBaseDto.
        :type: list[MapiPropertyDto]
        """
        self._properties = properties

    @property
    def discriminator(self) -> str:
        """
        Gets the discriminator of this MapiMessageItemBaseDto.

        :return: The discriminator of this MapiMessageItemBaseDto.
        :rtype: str
        """
        return self.__class__.__name__

    @discriminator.setter
    def discriminator(self, discriminator: str):
        """
        Sets the discriminator of this MapiMessageItemBaseDto.

        :param discriminator: The discriminator of this MapiMessageItemBaseDto.
        :type: str
        """
        pass    # setter is ignored for discriminator property

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MapiMessageItemBaseDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="StorageFile.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

import pprint
import re
import six
from typing import List, Set, Dict, Tuple, Optional
from datetime import datetime


class StorageFile(object):
    """File or folder information
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'is_folder': 'bool',
        'modified_date': 'datetime',
        'size': 'int',
        'path': 'str'
    }

    attribute_map = {
        'name': 'name',
        'is_folder': 'isFolder',
        'modified_date': 'modifiedDate',
        'size': 'size',
        'path': 'path'
    }

    def __init__(self, name: str = None, is_folder: bool = None, modified_date: datetime = None, size: int = None, path: str = None):
        """
        File or folder information
        :param name: File or folder name.
        :type name: str
        :param is_folder: True if it is a folder.
        :type is_folder: bool
        :param modified_date: File or folder last modified DateTime.
        :type modified_date: datetime
        :param size: File or folder size.
        :type size: int
        :param path: File or folder path.
        :type path: str
        """

        self._name = None
        self._is_folder = None
        self._modified_date = None
        self._size = None
        self._path = None

        if name is not None:
            self.name = name
        if is_folder is not None:
            self.is_folder = is_folder
        if modified_date is not None:
            self.modified_date = modified_date
        if size is not None:
            self.size = size
        if path is not None:
            self.path = path


    @property
    def name(self) -> str:
        """
        File or folder name.

        :return: The name of this StorageFile.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name: str):
        """
        File or folder name.

        :param name: The name of this StorageFile.
        :type: str
        """
        self._name = name

    @property
    def is_folder(self) -> bool:
        """
        True if it is a folder.

        :return: The is_folder of this StorageFile.
        :rtype: bool
        """
        return self._is_folder

    @is_folder.setter
    def is_folder(self, is_folder: bool):
        """
        True if it is a folder.

        :param is_folder: The is_folder of this StorageFile.
        :type: bool
        """
        if is_folder is None:
            raise ValueError("Invalid value for `is_folder`, must not be `None`")
        self._is_folder = is_folder

    @property
    def modified_date(self) -> datetime:
        """
        File or folder last modified DateTime.

        :return: The modified_date of this StorageFile.
        :rtype: datetime
        """
        return self._modified_date

    @modified_date.setter
    def modified_date(self, modified_date: datetime):
        """
        File or folder last modified DateTime.

        :param modified_date: The modified_date of this StorageFile.
        :type: datetime
        """
        self._modified_date = modified_date

    @property
    def size(self) -> int:
        """
        File or folder size.

        :return: The size of this StorageFile.
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size: int):
        """
        File or folder size.

        :param size: The size of this StorageFile.
        :type: int
        """
        if size is None:
            raise ValueError("Invalid value for `size`, must not be `None`")
        self._size = size

    @property
    def path(self) -> str:
        """
        File or folder path.

        :return: The path of this StorageFile.
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path: str):
        """
        File or folder path.

        :param path: The path of this StorageFile.
        :type: str
        """
        self._path = path

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StorageFile):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="CalendarReminder.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

import pprint
import re
import six
from typing import List, Set, Dict, Tuple, Optional
from datetime import datetime

from AsposeEmailCloudSdk.models.reminder_attendee import ReminderAttendee
from AsposeEmailCloudSdk.models.reminder_trigger import ReminderTrigger


class CalendarReminder(object):
    """Provides a grouping of component properties that define an alarm.             
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'action': 'str',
        'attachments': 'list[str]',
        'attendees': 'list[ReminderAttendee]',
        'description': 'str',
        'duration': 'int',
        'repeat': 'int',
        'summary': 'str',
        'trigger': 'ReminderTrigger'
    }

    attribute_map = {
        'action': 'action',
        'attachments': 'attachments',
        'attendees': 'attendees',
        'description': 'description',
        'duration': 'duration',
        'repeat': 'repeat',
        'summary': 'summary',
        'trigger': 'trigger'
    }

    def __init__(self, action: str = None, attachments: List[str] = None, attendees: List[ReminderAttendee] = None, description: str = None, duration: int = None, repeat: int = None, summary: str = None, trigger: ReminderTrigger = None):
        """
        Provides a grouping of component properties that define an alarm.             
        :param action: Defines the action to be invoked when an alarm is triggered. Enum, available values: Audio, Display, Email, Procedure, None
        :type action: str
        :param attachments: Collection of Reminder Attachments. Could be an absolute URI or Base64 string representation of attachment content             
        :type attachments: List[str]
        :param attendees: Contains collection of ReminderAttendee objects.             
        :type attendees: List[ReminderAttendee]
        :param description: Provides a more complete description of the alarm.
        :type description: str
        :param duration: Specifies the delay period in ticks, after which the alarm will repeat.             
        :type duration: int
        :param repeat: Defines the number of time the alarm should be repeated, after the initial trigger.             
        :type repeat: int
        :param summary: Defines a short summary or subject for the alarm.
        :type summary: str
        :param trigger: Specifies when an alarm will trigger.
        :type trigger: ReminderTrigger
        """

        self._action = None
        self._attachments = None
        self._attendees = None
        self._description = None
        self._duration = None
        self._repeat = None
        self._summary = None
        self._trigger = None

        if action is not None:
            self.action = action
        if attachments is not None:
            self.attachments = attachments
        if attendees is not None:
            self.attendees = attendees
        if description is not None:
            self.description = description
        if duration is not None:
            self.duration = duration
        if repeat is not None:
            self.repeat = repeat
        if summary is not None:
            self.summary = summary
        if trigger is not None:
            self.trigger = trigger


    @property
    def action(self) -> str:
        """
        Defines the action to be invoked when an alarm is triggered. Enum, available values: Audio, Display, Email, Procedure, None

        :return: The action of this CalendarReminder.
        :rtype: str
        """
        return self._action

    @action.setter
    def action(self, action: str):
        """
        Defines the action to be invoked when an alarm is triggered. Enum, available values: Audio, Display, Email, Procedure, None

        :param action: The action of this CalendarReminder.
        :type: str
        """
        if action is None:
            raise ValueError("Invalid value for `action`, must not be `None`")
        self._action = action

    @property
    def attachments(self) -> List[str]:
        """
        Collection of Reminder Attachments. Could be an absolute URI or Base64 string representation of attachment content             

        :return: The attachments of this CalendarReminder.
        :rtype: list[str]
        """
        return self._attachments

    @attachments.setter
    def attachments(self, attachments: List[str]):
        """
        Collection of Reminder Attachments. Could be an absolute URI or Base64 string representation of attachment content             

        :param attachments: The attachments of this CalendarReminder.
        :type: list[str]
        """
        self._attachments = attachments

    @property
    def attendees(self) -> List[ReminderAttendee]:
        """
        Contains collection of ReminderAttendee objects.             

        :return: The attendees of this CalendarReminder.
        :rtype: list[ReminderAttendee]
        """
        return self._attendees

    @attendees.setter
    def attendees(self, attendees: List[ReminderAttendee]):
        """
        Contains collection of ReminderAttendee objects.             

        :param attendees: The attendees of this CalendarReminder.
        :type: list[ReminderAttendee]
        """
        self._attendees = attendees

    @property
    def description(self) -> str:
        """
        Provides a more complete description of the alarm.

        :return: The description of this CalendarReminder.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description: str):
        """
        Provides a more complete description of the alarm.

        :param description: The description of this CalendarReminder.
        :type: str
        """
        self._description = description

    @property
    def duration(self) -> int:
        """
        Specifies the delay period in ticks, after which the alarm will repeat.             

        :return: The duration of this CalendarReminder.
        :rtype: int
        """
        return self._duration

    @duration.setter
    def duration(self, duration: int):
        """
        Specifies the delay period in ticks, after which the alarm will repeat.             

        :param duration: The duration of this CalendarReminder.
        :type: int
        """
        self._duration = duration

    @property
    def repeat(self) -> int:
        """
        Defines the number of time the alarm should be repeated, after the initial trigger.             

        :return: The repeat of this CalendarReminder.
        :rtype: int
        """
        return self._repeat

    @repeat.setter
    def repeat(self, repeat: int):
        """
        Defines the number of time the alarm should be repeated, after the initial trigger.             

        :param repeat: The repeat of this CalendarReminder.
        :type: int
        """
        if repeat is None:
            raise ValueError("Invalid value for `repeat`, must not be `None`")
        self._repeat = repeat

    @property
    def summary(self) -> str:
        """
        Defines a short summary or subject for the alarm.

        :return: The summary of this CalendarReminder.
        :rtype: str
        """
        return self._summary

    @summary.setter
    def summary(self, summary: str):
        """
        Defines a short summary or subject for the alarm.

        :param summary: The summary of this CalendarReminder.
        :type: str
        """
        self._summary = summary

    @property
    def trigger(self) -> ReminderTrigger:
        """
        Specifies when an alarm will trigger.

        :return: The trigger of this CalendarReminder.
        :rtype: ReminderTrigger
        """
        return self._trigger

    @trigger.setter
    def trigger(self, trigger: ReminderTrigger):
        """
        Specifies when an alarm will trigger.

        :param trigger: The trigger of this CalendarReminder.
        :type: ReminderTrigger
        """
        self._trigger = trigger

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CalendarReminder):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="MapiCalendarTimeZoneInfoDto.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

import pprint
import re
import six
from typing import List, Set, Dict, Tuple, Optional
from datetime import datetime

from AsposeEmailCloudSdk.models.mapi_calendar_time_zone_rule_dto import MapiCalendarTimeZoneRuleDto


class MapiCalendarTimeZoneInfoDto(object):
    """Represents the mapi calendar time zone rule.             
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'bias': 'int',
        'daylight_bias': 'int',
        'daylight_date': 'MapiCalendarTimeZoneRuleDto',
        'standard_bias': 'int',
        'standard_date': 'MapiCalendarTimeZoneRuleDto',
        'time_zone_flags': 'list[str]',
        'year': 'int'
    }

    attribute_map = {
        'bias': 'bias',
        'daylight_bias': 'daylightBias',
        'daylight_date': 'daylightDate',
        'standard_bias': 'standardBias',
        'standard_date': 'standardDate',
        'time_zone_flags': 'timeZoneFlags',
        'year': 'year'
    }

    def __init__(self, bias: int = None, daylight_bias: int = None, daylight_date: MapiCalendarTimeZoneRuleDto = None, standard_bias: int = None, standard_date: MapiCalendarTimeZoneRuleDto = None, time_zone_flags: List[str] = None, year: int = None):
        """
        Represents the mapi calendar time zone rule.             
        :param bias: Time zone's offset in minutes from UTC.             
        :type bias: int
        :param daylight_bias: Offset in minutes from lBias during daylight saving time.             
        :type daylight_bias: int
        :param daylight_date: Date and local time that indicate when to begin using the DaylightBias.             
        :type daylight_date: MapiCalendarTimeZoneRuleDto
        :param standard_bias: Offset in minutes from lBias during standard time.             
        :type standard_bias: int
        :param standard_date: Date and local time that indicate when to begin using the StandardBias.             
        :type standard_date: MapiCalendarTimeZoneRuleDto
        :param time_zone_flags: Individual bit flags that specify information about this TimeZoneRule.             
        :type time_zone_flags: List[str]
        :param year: Year in which this rule is scheduled to take effect.             
        :type year: int
        """

        self._bias = None
        self._daylight_bias = None
        self._daylight_date = None
        self._standard_bias = None
        self._standard_date = None
        self._time_zone_flags = None
        self._year = None

        if bias is not None:
            self.bias = bias
        if daylight_bias is not None:
            self.daylight_bias = daylight_bias
        if daylight_date is not None:
            self.daylight_date = daylight_date
        if standard_bias is not None:
            self.standard_bias = standard_bias
        if standard_date is not None:
            self.standard_date = standard_date
        if time_zone_flags is not None:
            self.time_zone_flags = time_zone_flags
        if year is not None:
            self.year = year


    @property
    def bias(self) -> int:
        """
        Time zone's offset in minutes from UTC.             

        :return: The bias of this MapiCalendarTimeZoneInfoDto.
        :rtype: int
        """
        return self._bias

    @bias.setter
    def bias(self, bias: int):
        """
        Time zone's offset in minutes from UTC.             

        :param bias: The bias of this MapiCalendarTimeZoneInfoDto.
        :type: int
        """
        if bias is None:
            raise ValueError("Invalid value for `bias`, must not be `None`")
        self._bias = bias

    @property
    def daylight_bias(self) -> int:
        """
        Offset in minutes from lBias during daylight saving time.             

        :return: The daylight_bias of this MapiCalendarTimeZoneInfoDto.
        :rtype: int
        """
        return self._daylight_bias

    @daylight_bias.setter
    def daylight_bias(self, daylight_bias: int):
        """
        Offset in minutes from lBias during daylight saving time.             

        :param daylight_bias: The daylight_bias of this MapiCalendarTimeZoneInfoDto.
        :type: int
        """
        if daylight_bias is None:
            raise ValueError("Invalid value for `daylight_bias`, must not be `None`")
        self._daylight_bias = daylight_bias

    @property
    def daylight_date(self) -> MapiCalendarTimeZoneRuleDto:
        """
        Date and local time that indicate when to begin using the DaylightBias.             

        :return: The daylight_date of this MapiCalendarTimeZoneInfoDto.
        :rtype: MapiCalendarTimeZoneRuleDto
        """
        return self._daylight_date

    @daylight_date.setter
    def daylight_date(self, daylight_date: MapiCalendarTimeZoneRuleDto):
        """
        Date and local time that indicate when to begin using the DaylightBias.             

        :param daylight_date: The daylight_date of this MapiCalendarTimeZoneInfoDto.
        :type: MapiCalendarTimeZoneRuleDto
        """
        self._daylight_date = daylight_date

    @property
    def standard_bias(self) -> int:
        """
        Offset in minutes from lBias during standard time.             

        :return: The standard_bias of this MapiCalendarTimeZoneInfoDto.
        :rtype: int
        """
        return self._standard_bias

    @standard_bias.setter
    def standard_bias(self, standard_bias: int):
        """
        Offset in minutes from lBias during standard time.             

        :param standard_bias: The standard_bias of this MapiCalendarTimeZoneInfoDto.
        :type: int
        """
        if standard_bias is None:
            raise ValueError("Invalid value for `standard_bias`, must not be `None`")
        self._standard_bias = standard_bias

    @property
    def standard_date(self) -> MapiCalendarTimeZoneRuleDto:
        """
        Date and local time that indicate when to begin using the StandardBias.             

        :return: The standard_date of this MapiCalendarTimeZoneInfoDto.
        :rtype: MapiCalendarTimeZoneRuleDto
        """
        return self._standard_date

    @standard_date.setter
    def standard_date(self, standard_date: MapiCalendarTimeZoneRuleDto):
        """
        Date and local time that indicate when to begin using the StandardBias.             

        :param standard_date: The standard_date of this MapiCalendarTimeZoneInfoDto.
        :type: MapiCalendarTimeZoneRuleDto
        """
        self._standard_date = standard_date

    @property
    def time_zone_flags(self) -> List[str]:
        """
        Individual bit flags that specify information about this TimeZoneRule.              Items: Enumerates the individual bit flags that specify information about TimeZoneRule Enum, available values: TzRuleFlagRecurCurrentTzReg, TzRuleFlagEffectiveTzReg

        :return: The time_zone_flags of this MapiCalendarTimeZoneInfoDto.
        :rtype: list[str]
        """
        return self._time_zone_flags

    @time_zone_flags.setter
    def time_zone_flags(self, time_zone_flags: List[str]):
        """
        Individual bit flags that specify information about this TimeZoneRule.              Items: Enumerates the individual bit flags that specify information about TimeZoneRule Enum, available values: TzRuleFlagRecurCurrentTzReg, TzRuleFlagEffectiveTzReg

        :param time_zone_flags: The time_zone_flags of this MapiCalendarTimeZoneInfoDto.
        :type: list[str]
        """
        self._time_zone_flags = time_zone_flags

    @property
    def year(self) -> int:
        """
        Year in which this rule is scheduled to take effect.             

        :return: The year of this MapiCalendarTimeZoneInfoDto.
        :rtype: int
        """
        return self._year

    @year.setter
    def year(self, year: int):
        """
        Year in which this rule is scheduled to take effect.             

        :param year: The year of this MapiCalendarTimeZoneInfoDto.
        :type: int
        """
        if year is None:
            raise ValueError("Invalid value for `year`, must not be `None`")
        self._year = year

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MapiCalendarTimeZoneInfoDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#  coding: utf-8
#  ----------------------------------------------------------------------------
#  <copyright company="Aspose" file="http_request.py">
#    Copyright (c) 2018-2020 Aspose Pty Ltd. All rights reserved.
#  </copyright>
#  <summary>
#    Permission is hereby granted, free of charge, to any person obtaining a
#   copy  of this software and associated documentation files (the "Software"),
#   to deal  in the Software without restriction, including without limitation
#   the rights  to use, copy, modify, merge, publish, distribute, sublicense,
#   and/or sell  copies of the Software, and to permit persons to whom the
#   Software is  furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all  copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM,  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#   DEALINGS IN THE SOFTWARE.
#  </summary>
#  ----------------------------------------------------------------------------

class HttpRequest(object):

    def __init__(self, resource_path, path_params, query_params, header_params,
                 form_params, body_params, files, collection_formats,
                 auth_settings, return_http_data_only=None,
                 preload_content=None, request_timeout=None):
        """Makes the HTTP request (synchronous) and returns deserialized data.

        To make an async request, set the async parameter.

        :param resource_path: Path to method endpoint.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be placed in the request
            header.
        :param form_params: Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param body_params: Request body.
        :param files: dict key -> filename, value -> filepath,
            for `multipart/form-data`.
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :param auth_settings: Auth Settings names for the request.
        :param return_http_data_only: response data without head status code
                                       and headers

        :param preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Request thread.
        """
        self.resource_path = resource_path
        self.path_params = path_params
        self.query_params = query_params
        self.header_params = header_params
        self.form_params = form_params
        self.body_params = body_params
        self.files = files
        self.collection_formats = collection_formats
        self.auth_settings = auth_settings
        self.return_http_data_only = return_http_data_only or True
        self.preload_content = preload_content or True
        self.request_timeout = request_timeout or ''
